# -*- coding: utf-8 -*-
'''
Created on 11.06.2012

@author: Jan Brohl
@license: Simplified BSD

Copyright (c) 2012, Jan Brohl
All rights reserved.

Redistribution and use in source and binary forms,
with or without modification,
are permitted provided that the following conditions are met:

    Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.

    Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
'''
from urlparse import urljoin
from urllib import quote
from scoutnet.util import geturl, convert, as_list
from datetime import date, time, datetime
from numbers import Integral
try:
    from json import dumps, loads, JSONDecoder
except:
    from simplejson  import dumps, loads, JSONDecoder


class RemoteObject(object):
    base_url = 'https://www.scoutnet.de/api/'

    @property
    def _api_pfad(self):
        """
        der unterpfad für das objekt
        """
        return urljoin(self.base_url, "%s/%i/" % (self.kind, int(self.id)))

    def __getattr__(self, name):
        """
        laden von kind-objekten - mit "query_"-prefix für komplexe anfragen
        "__" im name wird für die url mit "/" ersetzt aber das ist nicht so schön
        @todo: bessere lösung als "__"-ersetzung
        """
        unterpfad = name.replace("__", "/")
        if unterpfad.startswith("query_"):
            return (lambda query, *args:
                    self._query(unterpfad[6:], query, args))
        else:
            return self._load(urljoin(self._api_pfad, "%s/" % (unterpfad,)))

    def __init__(self, **kwargs):
        """
        Initialisierung die die kwargs zu attributen macht
        """
        for k, v in kwargs.items():
            setattr(self, k, v)

    def _query(self, unterpfad, query, args):
        """
        absetzten einer anfrage
        """
        qa = [query]
        if args:
            qa.append(args)
        return self._load(urljoin(self._api_pfad,
                                  "%s/?json=%s" % (unterpfad,
                                                    quote(dumps(qa))
                                                    )
                                  )
                          )

    def _load(self, url):
        """
        abrufen einer url und deserialisieren der json-antwort
        """
        s = geturl(url)
        obj = loads(s, object_hook=_object_hook)
        if isinstance(obj, Exception):
            raise obj
        return obj

    def _get(self, classname, ids):
        """
        abrufen mehrere objekte
        """
        return list(self._load(urljoin(self.base_url,
                                       "%s/%i/" % (classname, i)
                                       )
                               )
                    for i in ids)

    def __repr__(self):
        return "%s(%s)" % (self.__class__.__name__,
                           ", ".join("%s=%r" % (k, v)
                                     for k, v in self.__dict__.items()
                                     if not callable(v)
                                     )
                           )


class Group(RemoteObject):

    @property
    def _api_pfad(self):
        return urljoin(self.base_url, "%s/%i/" % (self.kind,
                                                  int(self.global_id)))


class Section(RemoteObject):
    pass


class Event(RemoteObject):
    pass


class ScoutnetException(Exception):
    pass


class API(RemoteObject):
    @property
    def _api_pfad(self):
        return self.base_url

    def event(self, event_id):
        """
        laden eines events
        """
        return self._get("event", [event_id])[0]

    def group(self, group_id):
        """
        laden einer group
        """
        return self._get("group", [group_id])[0]


def _object_hook(object_dict):
    """
    erstellen der objekte aus einem dict
    """
    kind = object_dict.get("kind", None)
    stuff = dict()
    for k, v in object_dict.items():
        if v is not None:
            if isinstance(v, basestring):
                v = convert(v)
            elif isinstance(v, dict):
                v = _object_hook(v)
        stuff[convert(k)] = v
    if kind == "group":
        return Group(**stuff)
    elif kind == "event":
        return Event(**stuff)
    elif kind == "exception":
        return ScoutnetException(stuff["message"])
    elif kind == "collection":
        return stuff["elements"]
    else:
        return as_list(stuff)
